import { Web3 } from './web3wrapper';
import { Ethereum } from '@tesseractjs/ethereum/esm';
import { Network } from './types';
import { Web3FallbackProvider, Web3NativeProvider, Web3OpenWalletProvider } from './providers';
Ethereum.addPlugin("Web3", (ethereum) => {
    function web3Constructor(rpcUrlOrNetId, rpcOptions) {
        let netId;
        let rpcUrl;
        if (typeof rpcUrlOrNetId === 'string') {
            rpcUrl = rpcUrlOrNetId;
        }
        else {
            netId = rpcUrlOrNetId;
            const rpcInfo = ethereum.Web3.rpcUrls[netId];
            if (rpcInfo) {
                if (typeof rpcInfo === 'string') {
                    rpcUrl = rpcInfo;
                }
                else {
                    rpcUrl = rpcInfo.url;
                    rpcOptions = rpcOptions || rpcInfo.options;
                }
            }
        }
        return Web3.create(ethereum.Web3.providers, ethereum.openWallet, netId, rpcUrl, rpcOptions);
    }
    web3Constructor.rpcUrls = {};
    web3Constructor.providers = [Web3NativeProvider, Web3OpenWalletProvider, Web3FallbackProvider];
    return web3Constructor;
});
export { Web3, Web3FallbackProvider, Web3NativeProvider, Web3OpenWalletProvider, Ethereum, Network };
export { Tesseract } from '@tesseractjs/core/esm';
