import { Ethereum } from '@tesseractjs/openwallet-ethereum/esm';
import { getChainId, promisifiedSend } from '../../rpc';
import { OpenWalletNodeProvider } from './node';
import { HANDLERS } from './handlers';
export class Web3OpenWalletProvider {
    constructor(netId, ethereum, provider) {
        this.hasClientWallet = true;
        this.netId = netId;
        this.chainId = getChainId(provider);
        this.provider = provider;
        this.ethereum = ethereum;
    }
    get connected() {
        return this.provider.connected;
    }
    supportsSubscriptions() {
        return this.provider.supportsSubscriptions();
    }
    send(payload, callback) {
        this.chainId
            .then(chainId => {
            const handler = HANDLERS[payload.method];
            if (handler) {
                return handler(this.ethereum, payload, this.netId, chainId, this.provider)
                    .then(result => ({ jsonrpc: payload.jsonrpc, id: payload.id, result }));
            }
            return promisifiedSend(this.provider, payload);
        })
            .then(response => {
            if (response.error)
                throw response.error;
            callback(null, response);
        })
            .catch(err => callback(err));
    }
    on(type, callback) {
        if (!this.supportsSubscriptions()) {
            throw new Error('Subscriptions is not supported');
        }
        this.provider.on(type, callback);
    }
    removeListener(type, callback) {
        if (!this.supportsSubscriptions()) {
            throw new Error('Subscriptions is not supported');
        }
        this.provider.removeListener(type, callback);
    }
    reset() {
        if (typeof this.provider.reset === 'function') {
            this.provider.reset();
        }
    }
    static async createNative(ethereum, netId) {
        const provider = await OpenWalletNodeProvider.create(ethereum, netId);
        return new Web3OpenWalletProvider(netId, ethereum, provider);
    }
    static async create({ openWallet, provider, netId }) {
        const ethereum = new Ethereum(openWallet);
        if (!openWallet.hasOpenWallet) {
            throw new Error('OpenWallet is not supported');
        }
        if (!await ethereum.isKeychainInstalled()) {
            throw new Error('OpenWallet does not support Ethereum keychain');
        }
        try {
            return await this.createNative(ethereum, netId);
        }
        catch (_a) {
            if (!provider) {
                throw new Error('Doesn\'t have any provider');
            }
            return new Web3OpenWalletProvider(netId, ethereum, provider);
        }
    }
}
