import { getNetId } from '../rpc';
export class Web3NativeProvider {
    has(target, p) {
        switch (p) {
            case 'hasClientWallet': return true;
            case 'supportsSubscriptions': return true;
            default: return Reflect.has(target, p);
        }
    }
    get(target, p, receiver) {
        switch (p) {
            case 'hasClientWallet': return true;
            case 'supportsSubscriptions': {
                const func = Reflect.get(target, p, receiver);
                if (typeof func === 'function') {
                    return func;
                }
                return () => Reflect.get(target, 'on', receiver) !== undefined;
            }
            default: return Reflect.get(target, p, receiver);
        }
    }
    set(target, p, value, receiver) {
        switch (p) {
            case 'hasClientWallet': return false;
            case 'supportsSubscriptions': return false;
            default: return Reflect.set(target, p, value, receiver);
        }
    }
    ownKeys(target) {
        return Reflect.ownKeys(target).concat(['hasClientWallet', 'supportsSubscriptions']);
    }
    static async _fromProvider(provider, netId) {
        const rpcNetId = await getNetId(provider);
        if (netId !== rpcNetId) {
            throw new Error('Provider has different netId');
        }
        return new Proxy(provider, new Web3NativeProvider());
    }
    static async create({ netId }) {
        if (!window) {
            throw new Error('Will work only in browser');
        }
        if (window.ethereum) {
            await window.ethereum.enable();
            return await this._fromProvider(window.ethereum, netId);
        }
        if (window.web3 && window.web3.currentProvider) {
            return await this._fromProvider(window.web3.currentProvider, netId);
        }
        throw new Error("Can't create provider");
    }
}
