import { Web3, Jsonrpc } from './libs';
export function promisifiedSend(provider, request) {
    return new Promise((resolve, reject) => {
        provider.send(request, (error, response) => {
            if (error) {
                reject(error);
            }
            else {
                resolve(response);
            }
        });
    });
}
export function getNetId(provider) {
    const request = Jsonrpc.toPayload('net_version', []);
    return promisifiedSend(provider, request)
        .then(response => {
        if (response.error) {
            throw response.error;
        }
        return parseInt(response.result, 10);
    });
}
export function getChainId(provider) {
    const request = Jsonrpc.toPayload('eth_chainId', []);
    return promisifiedSend(provider, request)
        .then(response => {
        if (response.error) {
            throw response.error;
        }
        return response.result;
    });
}
export function estimateGas(provider, tx) {
    const request = Jsonrpc.toPayload('eth_estimateGas', [tx]);
    return promisifiedSend(provider, request)
        .then(response => {
        if (response.error) {
            throw response.error;
        }
        return Web3.utils.toBN(response.result);
    });
}
export function getGasPrice(provider) {
    const request = Jsonrpc.toPayload('eth_gasPrice', []);
    return promisifiedSend(provider, request)
        .then(response => {
        if (response.error) {
            throw response.error;
        }
        return Web3.utils.toBN(response.result);
    });
}
export function getTransactionCount(provider, address) {
    const request = Jsonrpc.toPayload('eth_getTransactionCount', [address, 'pending']);
    return promisifiedSend(provider, request)
        .then(response => {
        if (response.error) {
            throw response.error;
        }
        return response.result;
    });
}
