"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var libs_1 = require("../../libs");
var OpenWalletNodeProvider = /** @class */ (function () {
    function OpenWalletNodeProvider(ethereum, netId, supportsSubscriptions) {
        this.subscriptions = {};
        this.subscribers = {};
        this.connected = true;
        this.ethereum = ethereum;
        this.netId = netId;
        this._supportsSubscriptions = supportsSubscriptions;
    }
    OpenWalletNodeProvider.prototype.supportsSubscriptions = function () {
        return this._supportsSubscriptions;
    };
    OpenWalletNodeProvider.prototype.emit = function (type, message) {
        var subscribers = this.subscribers[type];
        if (subscribers) {
            for (var _i = 0, subscribers_1 = subscribers; _i < subscribers_1.length; _i++) {
                var subs = subscribers_1[_i];
                subs(message);
            }
        }
    };
    OpenWalletNodeProvider.prototype.send = function (payload, callback) {
        var _this = this;
        var promise;
        if (payload.method.endsWith('_subscribe')) {
            var request = __assign({ networkId: this.netId }, payload);
            promise = this.ethereum.Node.subscribe(request)
                .then(function (subscription) {
                var subscriptionId = subscription.response;
                _this.subscriptions[subscriptionId] = subscription;
                subscription.on('message', function (message) {
                    _this.emit('data', message);
                });
                subscription.on('unsubscribed', function () {
                    delete _this.subscriptions[subscriptionId];
                });
                return subscription.response;
            });
        }
        else if (payload.method.endsWith('_unsubscribe')) {
            var subscription = this.subscriptions[payload.params[0]];
            if (subscription) {
                promise = subscription.unsubscribe({
                    networkId: this.netId,
                    method: 'eth_unsubscribe',
                    params: [subscription.response]
                });
            }
            else {
                promise = Promise.reject({ type: 'METHOD_NOT_FOUND', message: "method not found: " + payload.method, code: -32000 });
            }
        }
        else {
            var request = __assign({ networkId: this.netId }, payload);
            promise = this.ethereum.Node.send(request);
        }
        var id = typeof payload.id === 'number'
            ? payload.id
            : typeof payload.id === 'string' ? parseInt(payload.id, 10) : libs_1.Jsonrpc.messageId++;
        promise
            .then(function (result) { return callback(null, { id: id, jsonrpc: payload.jsonrpc, result: result }); })
            .catch(function (error) { return callback(null, { id: id, jsonrpc: payload.jsonrpc, error: error }); });
    };
    OpenWalletNodeProvider.prototype.on = function (type, callback) {
        var subs = this.subscribers[type] || [];
        subs.push(callback);
        this.subscribers[type] = subs;
    };
    OpenWalletNodeProvider.prototype.removeListener = function (type, callback) {
        var subs = this.subscribers[type] || [];
        var index = subs.indexOf(callback);
        if (index >= 0) {
            subs.splice(index, 1);
        }
        this.subscribers[type] = subs;
    };
    OpenWalletNodeProvider.prototype.reset = function () {
        this.subscribers = {};
        for (var id in this.subscriptions) {
            var subs = this.subscriptions[id];
            subs.unsubscribe({
                networkId: this.netId,
                method: 'eth_unsubscribe',
                params: [subs.response]
            });
        }
        this.subscriptions = {};
    };
    OpenWalletNodeProvider.create = function (ethereum, netId) {
        return __awaiter(this, void 0, void 0, function () {
            var supportedNetworks, supportsSubscriptions;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, ethereum.Node.canSend()];
                    case 1:
                        if (!(_a.sent())) {
                            throw new Error('Node API is not supported');
                        }
                        return [4 /*yield*/, ethereum.Node.supportedNetworks()];
                    case 2:
                        supportedNetworks = _a.sent();
                        if (supportedNetworks.indexOf(netId) < 0) {
                            throw new Error('Network is not supported: ' + netId);
                        }
                        return [4 /*yield*/, ethereum.Node.canSubscribe()];
                    case 3:
                        supportsSubscriptions = _a.sent();
                        return [2 /*return*/, new OpenWalletNodeProvider(ethereum, netId, supportsSubscriptions)];
                }
            });
        });
    };
    return OpenWalletNodeProvider;
}());
exports.OpenWalletNodeProvider = OpenWalletNodeProvider;
