"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var libs_1 = require("./libs");
function promisifiedSend(provider, request) {
    return new Promise(function (resolve, reject) {
        provider.send(request, function (error, response) {
            if (error) {
                reject(error);
            }
            else {
                resolve(response);
            }
        });
    });
}
exports.promisifiedSend = promisifiedSend;
function getNetId(provider) {
    var request = libs_1.Jsonrpc.toPayload('net_version', []);
    return promisifiedSend(provider, request)
        .then(function (response) {
        if (response.error) {
            throw response.error;
        }
        return parseInt(response.result, 10);
    });
}
exports.getNetId = getNetId;
function getChainId(provider) {
    var request = libs_1.Jsonrpc.toPayload('eth_chainId', []);
    return promisifiedSend(provider, request)
        .then(function (response) {
        if (response.error) {
            throw response.error;
        }
        return response.result;
    });
}
exports.getChainId = getChainId;
function estimateGas(provider, tx) {
    var request = libs_1.Jsonrpc.toPayload('eth_estimateGas', [tx]);
    return promisifiedSend(provider, request)
        .then(function (response) {
        if (response.error) {
            throw response.error;
        }
        return libs_1.Web3.utils.toBN(response.result);
    });
}
exports.estimateGas = estimateGas;
function getGasPrice(provider) {
    var request = libs_1.Jsonrpc.toPayload('eth_gasPrice', []);
    return promisifiedSend(provider, request)
        .then(function (response) {
        if (response.error) {
            throw response.error;
        }
        return libs_1.Web3.utils.toBN(response.result);
    });
}
exports.getGasPrice = getGasPrice;
function getTransactionCount(provider, address) {
    var request = libs_1.Jsonrpc.toPayload('eth_getTransactionCount', [address, 'pending']);
    return promisifiedSend(provider, request)
        .then(function (response) {
        if (response.error) {
            throw response.error;
        }
        return response.result;
    });
}
exports.getTransactionCount = getTransactionCount;
